/*
Tetris for Zx3
(C) 2010 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "display.h"

/*
The ZX3 seems to have a framebuffer consisting of layers. The layer the menu
system uses consists of an arry of 1024x240 (240 is a guess) bytes; each
byte is mapped to a pixel. Any piece of memory can be used as a
framebuffer, just write its address into _fbAddr (mapped in the link script).
This can be used to implement double buffering.
I have no idea yet where the color lookup-table is tho'.
*/

#define FB_W 1024
#define FB_H 320 //?

#define BLKH 10
#define BLKW BLKH*2


//ToDo: Rewrite to place 32bit words instead of 8bit chars
//Place a block at a certain place on the screen.
static void placeBlock(volatile char *fb, int col, int bx, int by) {
	unsigned int palette[8]={110, 15, 98, 47, 63, 78, 88, 120};
	int x,y;
	for (x=0; x<BLKW; x++) {
		for (y=0; y<BLKH; y++) {
			fb[(bx*BLKW+x+256)+(by*BLKH+y+20)*FB_W]=palette[col&7];
		}
	}
}

//Clear screen. Overwrite everything with '111', which maps to a 
//semi-transparent color.
void displayClear(volatile char *fb) {
	int x,y;
	volatile char *p=fb;
	for (y=0; y<FB_H; y++) {
		for (x=0; x<FB_W; x++) {
			*p=111;
			p++;
		}
	}
}

//Display a Tetris playing field
void display(volatile char* fb, playfield_t *field) {
	int x,y;
	int px,py;
	for (x=0; x<10; x++) {
		for (y=0; y<20; y++) {
			placeBlock(fb, field->field[x][y], x, y);
		}
	}

	for (x=0; x<4; x++) {
		for (y=0; y<4; y++) {
			px=x+field->currBlockX;
			py=y+field->currBlockY;
			if (field->currBlock[x][y]!=0) {
				if (px>=0 && py>=0 && px<10 && py<20) {
					placeBlock(fb, field->currBlock[x][y], px, py);
				}
			}
		}
	}

}
