/*
Tetris for Zx3
(C) 2010 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "os.h"

#include "playfield.h"
#include "display.h"

playfield_t field;
playfield_t testfield;

//some stupid random number routines, borrowed from glibcs rand() and friends
static unsigned int randSeed=0;

void randAddEnt(int chr) {
	randSeed+=chr;
}

int rand() {
	randSeed = randSeed * 1103515245 + 12345;
	return randSeed%65536;
}

//Main routine
int main() {
	volatile char *fb1, *fb2, *tmpfb;
	unsigned int ev;
	int x,y;
	int dead=0;
	int mustFixBlk;
	//We can use any address as the framebuffer address, but I decided to recycle the addresses the original fw uses.
	fb1=(void*)0xc1136340;
	fb2=(void*)0xc1136340+(1024*256); //guess

	printf("Tetris init");
	fieldClear(&field);
	fieldSelectBlk(&field, 2);
	printf("Tetris init done.");

	displayClear(fb1);
	displayClear(fb2);

	do {
		printf("Waiting for event. bx=%i, by=%i", field.currBlockX, field.currBlockY);
		getAppEvent(&ev);
		printf("AppEvent: %x",ev);
		fieldDup(&testfield, &field);
		mustFixBlk=0;

		randAddEnt(ev); //kick the rng!

		if ((ev&APPEV_MASK_TYPE)==APPEV_TYPE_TIMER) {
			if ((ev&APPEV_MASK_VAL)==1) {
				testfield.currBlockY++;
				if (fieldIsPossible(&testfield)) {
					fieldDup(&field, &testfield);
				} else {
					mustFixBlk=1;
				}
			}
		}
		if ((ev&APPEV_MASK_TYPE)==APPEV_TYPE_KEYPRESS && !dead) {
			if ((ev&APPEV_MASK_VAL)==APPEV_KEY_DOWN) {
				while(fieldIsPossible(&testfield)) {
					fieldDup(&field, &testfield);
					testfield.currBlockY++;
				}
				mustFixBlk=1;
			}
			if ((ev&APPEV_MASK_VAL)==APPEV_KEY_LEFT) {
				testfield.currBlockX--;
				if (fieldIsPossible(&testfield)) {
					fieldDup(&field, &testfield);
				}
			}
			if ((ev&APPEV_MASK_VAL)==APPEV_KEY_RIGHT) {
				testfield.currBlockX++;
				if (fieldIsPossible(&testfield)) {
					fieldDup(&field, &testfield);
				}
			}
			if ((ev&APPEV_MASK_VAL)==APPEV_KEY_SELECT) {
				fieldRotateBlk(&testfield, 1);
				if (fieldIsPossible(&testfield)) {
					fieldDup(&field, &testfield);
				}
			}
		}
		if (mustFixBlk && dead==0) {
			printf("Fixing block, giving out new one.");
			fieldFixBlock(&field);
			fieldSelectBlk(&field, rand()%7);
			if (!fieldIsPossible(&field)) {
				printf("Ahrg! Player is dead.");
				//Player died! Clear currblk and fill field so he can't do anything but quit
				for (x=0; x<4; x++) {
					for (y=0; y<4; y++) {
						field.currBlock[x][y]=0;
					}
				}
				for (x=0; x<10; x++) {
					for (y=0; y<20; y++) {
						field.field[x][y]=1;
					}
				}
				dead=1;
			}
		}

		display(fb1, &field);
		_fbAddr=fb1;
		tmpfb=fb2; fb2=fb1; fb1=tmpfb;

	} while(ev!=(APPEV_TYPE_KEYPRESS|APPEV_KEY_POWER));
	return 0;
}
